#!/bin/bash
#
# Validate that the current version of the generated cache_size files match the output
# generated by sizegen.
#
# This is used in Travis to verify that the currently committed version was
# generated with the proper cache_size files.

source build.env

TMP="/tmp/cached_size.$$.go"
ALL_FILES=$(find . -name "cached_size.go")

set +e

goimports -local vitess.io/vitess -w $ALL_FILES

for SRC in $ALL_FILES
do
  TMP="/tmp/"$(echo "$SRC" | sed 's/\//_/g' | sed "s/cached_size.go/cached_size_$$.go/g")
  mv "$SRC" "$TMP"
done

make sizegen

STATUS=0

for SRC in $ALL_FILES
do
  TMP="/tmp/"$(echo "$SRC" | sed 's/\//_/g' | sed "s/cached_size.go/cached_size_$$.go/g")

  if [ ! -f "$SRC" ]; then
    mv "$TMP" "$SRC"
    continue
  fi

  if ! diff -q "$SRC" "$TMP" > /dev/null ; then
    echo "ERROR: Regenerated file for $SRC does not match the current version:"
    diff -u "$SRC" "$TMP"

    echo
    echo "Please re-run 'make sizegen' to generate."
    STATUS=1
  fi
  mv "$TMP" "$SRC"
done

exit $STATUS

